var app = (function (exports) {
  'use strict';

  function isWindow(obj) {
    return obj != null && obj === obj.window;
  }

  function find(selector) {
    var context = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;

    if (selector instanceof HTMLElement || selector instanceof Node || isWindow(selector)) {
      return [selector];
    } else if (selector instanceof NodeList) {
      return [].slice.call(selector);
    } else if (typeof selector === 'string') {
      var startElement = context ? find(context)[0] : document;
      return [].slice.call(startElement.querySelectorAll(selector));
    }

    return [];
  }

  function on(selector, event, cb) {
    var capture = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;

    if (Array.isArray(selector)) {
      selector.forEach(function (item) {
        return on(item, event, cb, capture);
      });
      return;
    }

    var data = {
      cb: cb,
      capture: capture
    };

    if (!window._domassistevents) {
      window._domassistevents = {};
    }

    window._domassistevents["_".concat(event)] = data;
    var el = find(selector);

    if (el.length) {
      el.forEach(function (item) {
        item.addEventListener(event, cb, capture);
      });
    }
  }

  var NativeCustomEvent = window.CustomEvent; //
  // Check for the usage of native support for CustomEvents which is lacking
  // completely on IE.
  //

  function canIuseNativeCustom() {
    try {
      var p = new NativeCustomEvent('t', {
        detail: {
          a: 'b'
        }
      });
      return p.type === 't' && p.detail.a === 'b';
    } catch (e) {
      return false;
    }
  } // Lousy polyfill for the Custom Event constructor for IE.


  var IECustomEvent = function CustomEvent(type, params) {
    var e = document.createEvent('CustomEvent');

    if (params) {
      e.initCustomEvent(type, params.bubbles, params.cancelable, params.detail);
    } else {
      e.initCustomEvent(type, false, false, undefined);
    }

    return e;
  };

  var DomassistCustomEvent = canIuseNativeCustom() ? NativeCustomEvent : IECustomEvent;
  var SCROLLABLE_CONTAINER;

  function getScrollableContainer() {
    if (SCROLLABLE_CONTAINER) {
      return SCROLLABLE_CONTAINER;
    }

    var documentElement = window.document.documentElement;
    var scrollableContainer;
    documentElement.scrollTop = 1;

    if (documentElement.scrollTop === 1) {
      documentElement.scrollTop = 0;
      scrollableContainer = documentElement;
    } else {
      scrollableContainer = document.body;
    }

    SCROLLABLE_CONTAINER = scrollableContainer;
    return scrollableContainer;
  }

  SCROLLABLE_CONTAINER = getScrollableContainer();

  var debounce = function debounce(func, wait, immediate) {
    var timeout;
    return function () {
      var context = this;
      var args = arguments; // eslint-disable-line prefer-rest-params

      var later = function later() {
        timeout = null;

        if (!immediate) {
          func.apply(context, args);
        }
      };

      var callNow = immediate && !timeout;
      clearTimeout(timeout);
      timeout = setTimeout(later, wait);

      if (callNow) {
        func.apply(context, args);
      }
    };
  };
  /* eslint-env browser */


  var scrollHandlers = [];
  on(window, 'scroll', function (event) {
    scrollHandlers.forEach(function (listener) {
      return listener.handler.call(null, event);
    });
  });
  var index = {
    on: function on(callback, wait, immediate) {
      var handler = callback;

      if (wait) {
        handler = debounce(callback, wait, immediate);
      }

      scrollHandlers.push({
        handler: handler,
        callback: callback,
        wait: wait,
        immediate: immediate
      });
    },
    off: function off(callback, wait, immediate) {
      if (!callback) {
        scrollHandlers.length = 0;
      } else {
        scrollHandlers = scrollHandlers.filter(function (listener) {
          return listener.callback !== callback && listener.wait !== wait && listener.immediate !== immediate;
        });
      }
    }
  };

  function _typeof(obj) {
    if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") {
      _typeof = function (obj) {
        return typeof obj;
      };
    } else {
      _typeof = function (obj) {
        return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
      };
    }

    return _typeof(obj);
  }

  function _typeof$1(obj) {
    if (typeof Symbol === "function" && _typeof(Symbol.iterator) === "symbol") {
      _typeof$1 = function _typeof$1(obj) {
        return _typeof(obj);
      };
    } else {
      _typeof$1 = function _typeof$1(obj) {
        return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : _typeof(obj);
      };
    }

    return _typeof$1(obj);
  }

  function isWindow$1(obj) {
    return obj != null && obj === obj.window;
  }

  function find$1(selector) {
    var context = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;

    if (selector instanceof HTMLElement || selector instanceof Node || isWindow$1(selector)) {
      return [selector];
    } else if (selector instanceof NodeList) {
      return [].slice.call(selector);
    } else if (typeof selector === 'string') {
      var startElement = context ? find$1(context)[0] : document;
      return [].slice.call(startElement.querySelectorAll(selector));
    }

    return [];
  }

  function on$1(selector, event, cb) {
    var capture = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;

    if (Array.isArray(selector)) {
      selector.forEach(function (item) {
        return on$1(item, event, cb, capture);
      });
      return;
    }

    var data = {
      cb: cb,
      capture: capture
    };

    if (!window._domassistevents) {
      window._domassistevents = {};
    }

    window._domassistevents["_".concat(event)] = data;
    var el = find$1(selector);

    if (el.length) {
      el.forEach(function (item) {
        item.addEventListener(event, cb, capture);
      });
    }
  }

  function matches(el, selector) {
    var proto = Element.prototype;
    var match = false;
    var prefixes = ['matches', 'matchesSelector', 'webkitMatchesSelector', 'mozMatchesSelector', 'msMatchesSelector', 'oMatchesSelector'];
    prefixes.forEach(function (prefix) {
      if (proto.hasOwnProperty(prefix)) {
        match = proto[prefix];
      }
    });

    if (match) {
      return el ? match.call(el, selector) : null;
    }
  }

  function closest(el, selector) {
    if (matches(el, selector)) {
      return el;
    }

    var parent = el.parentElement ? el.parentElement : el.parentNode;

    while (parent.parentElement && !matches(parent, selector)) {
      parent = parent.parentElement;
    }

    return matches(parent, selector) ? parent : null;
  }

  var NativeCustomEvent$1 = window.CustomEvent; //
  // Check for the usage of native support for CustomEvents which is lacking
  // completely on IE.
  //

  function canIuseNativeCustom$1() {
    try {
      var p = new NativeCustomEvent$1('t', {
        detail: {
          a: 'b'
        }
      });
      return p.type === 't' && p.detail.a === 'b';
    } catch (e) {
      return false;
    }
  } // Lousy polyfill for the Custom Event constructor for IE.


  var IECustomEvent$1 = function CustomEvent(type, params) {
    var e = document.createEvent('CustomEvent');

    if (params) {
      e.initCustomEvent(type, params.bubbles, params.cancelable, params.detail);
    } else {
      e.initCustomEvent(type, false, false, undefined);
    }

    return e;
  };

  var DomassistCustomEvent$1 = canIuseNativeCustom$1() ? NativeCustomEvent$1 : IECustomEvent$1;
  var SCROLLABLE_CONTAINER$1;

  function getScrollableContainer$1() {
    if (SCROLLABLE_CONTAINER$1) {
      return SCROLLABLE_CONTAINER$1;
    }

    var documentElement = window.document.documentElement;
    var scrollableContainer;
    documentElement.scrollTop = 1;

    if (documentElement.scrollTop === 1) {
      documentElement.scrollTop = 0;
      scrollableContainer = documentElement;
    } else {
      scrollableContainer = document.body;
    }

    SCROLLABLE_CONTAINER$1 = scrollableContainer;
    return scrollableContainer;
  }

  SCROLLABLE_CONTAINER$1 = getScrollableContainer$1();

  var setupReady = function setupReady(callbacks) {
    return function (callback) {
      callbacks.push(callback);

      function execute() {
        while (callbacks.length) {
          var fn = callbacks.shift();

          if (typeof fn === 'function') {
            fn();
          }
        }
      }

      function loaded() {
        document.removeEventListener('DOMContentLoaded', loaded);
        execute();
      }

      setTimeout(function () {
        if (document.readyState !== 'loading') {
          return execute();
        }
      }, 0);
      document.addEventListener('DOMContentLoaded', loaded);
    };
  };

  var ready = setupReady([]);

  function _typeof$1$1(obj) {
    if (typeof Symbol === "function" && _typeof$1(Symbol.iterator) === "symbol") {
      _typeof$1$1 = function _typeof$1$1(obj) {
        return _typeof$1(obj);
      };
    } else {
      _typeof$1$1 = function _typeof$1$1(obj) {
        return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : _typeof$1(obj);
      };
    }

    return _typeof$1$1(obj);
  }
  /* eslint-disable no-restricted-syntax, guard-for-in, prefer-rest-params, no-use-before-define */


  var aug = function aug() {
    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    args.unshift(false);
    return merge.apply(null, args);
  };

  aug.defaults = function () {
    for (var _len2 = arguments.length, args = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
      args[_key2] = arguments[_key2];
    }

    args.unshift(true);
    return merge.apply(null, args);
  }; // first arg is 'true' if using defaults-only version
  // first arg is 'false' if just doing deep merge:


  function merge() {
    for (var _len3 = arguments.length, args = new Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
      args[_key3] = arguments[_key3];
    }

    var useDefaults = args[0];
    var destObject = {}; // for each object in the rest of the argument list:

    args.slice(1, args.length).forEach(function (prop) {
      // for each property in the current object:
      for (var propName in prop) {
        // default-only mode skips the property if it's not present in the first object
        if (useDefaults && args[1][propName] === undefined) {
          return;
        } // get the value we are going to assign to the dest object:


        var propValue = prop[propName]; // if that value is an array just assign it to the dest object:

        if (Array.isArray(propValue)) {
          destObject[propName] = propValue;
          continue;
        } // if the source and destination values are both objects then recursively merge them:


        if (_typeof$1$1(propValue) === 'object' && _typeof$1$1(destObject[propName]) === 'object') {
          // get the right merging function for the recursive merge:
          var merger = useDefaults ? aug.defaults : aug;
          destObject[propName] = merger(destObject[propName], propValue);
          continue;
        } // otherwise just assign the value to the destination object:


        destObject[propName] = propValue;
      }
    });
    return destObject;
  }

  var GATrack = {
    sendEvent: function sendEvent(category, action, label) {
      var callback = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : null;
      var timeout = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : 1000;

      if (GATrack.prefix) {
        category = "".concat(GATrack.prefix, "-").concat(category);
      }

      var onHit = function onHit() {};

      GATrack.log(category, action, label);

      if (!GATrack.isEnabled) {
        if (typeof callback === 'function') {
          callback();
        }

        return GATrack;
      }

      if (typeof callback === 'function') {
        var timeoutId;

        onHit = function onHit() {
          if (!timeoutId) {
            return;
          }

          clearTimeout(timeoutId);
          timeoutId = null;
          callback();
        };

        timeoutId = setTimeout(onHit, timeout);
      }

      if (GATrack.isGAQ) {
        _gaq.push(['_trackEvent', category, action, label, null, false]);

        _gaq.push(onHit);
      } else if (GATrack.isGTag) {
        // Gtag check needs to go before since gtag creates a ga variable
        var payload = {
          event_category: category,
          event_label: label,
          event_callback: onHit
        };
        GATrack.sendData('event', action, payload);
      } else if (GATrack.isGA) {
        var options = {
          transport: 'beacon',
          hitCallback: onHit
        };
        GATrack.sendData('send', 'event', category, action, label, options);
      }
    },
    send: function send() {
      // eslint-disable-next-line prefer-rest-params
      var args = Array.prototype.slice.call(arguments);

      if (GATrack.isGA) {
        args.unshift('send');
      }

      GATrack.sendData.apply(null, args);
    },
    sendData: function sendData() {
      if (!GATrack.isEnabled) {
        return GATrack;
      } // eslint-disable-next-line prefer-rest-params


      var args = Array.prototype.slice.call(arguments);

      if (GATrack.isGA) {
        ga.apply(null, args);
      } else if (GATrack.isGTag) {
        gtag.apply(null, args);
      }
    },
    getData: function getData(element) {
      var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
      var href = element.dataset.gaTrackHref || element.getAttribute('href');
      var category = element.dataset.gaTrack || options.category || 'ga-track';
      var label = element.dataset.gaTrackLabel || options.label || href;
      var action = element.dataset.gaTrackAction || options.action || element.textContent.trim();
      return {
        href: href,
        category: category,
        label: label,
        action: action
      };
    },
    track: function track(element) {
      var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      if (Array.isArray(element)) {
        element.forEach(function (data) {
          find$1(data.element).forEach(function (el) {
            GATrack.track(el, data);
          });
        });
        return;
      }

      if (typeof element.dataset.gaTrackInitialised !== 'undefined') {
        return;
      }

      element.dataset.gaTrackInitialised = 'true';
      options = aug({}, GATrack.defaults, options);
      this.log('tracking', element, options);
      on$1(element, 'click', function (event) {
        GATrack.onTrackedClick(element, event, options);
      });
    },
    onTrackedClick: function onTrackedClick(element, event, options) {
      var data = GATrack.getData(element, options);
      var target = element.getAttribute('target');
      var callback = null;

      if (element.dataset.gaTrackHref === 'false') {
        event.preventDefault();
      } else if (data.href && !event.metaKey && event.which === 1 && target !== '_blank') {
        event.preventDefault();

        callback = function callback() {
          window.location = data.href;
        };
      }

      GATrack.sendEvent(data.category, data.action, data.label, callback, options.timeout);
    },
    autotrack: function autotrack() {
      if (GATrack.autotracking === true) {
        return;
      }

      GATrack.autotracking = true;
      var selector = '[data-ga-track]';
      on$1(document.body, 'click', function (event) {
        var element = event.target;

        if (!matches(element, selector)) {
          element = closest(element, selector);
        }

        if (element) {
          GATrack.onTrackedClick(element, event, GATrack.defaults);
        }
      });
    },
    log: function log() {
      if (GATrack.debug) {
        var _console;

        for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
          args[_key] = arguments[_key];
        }

        (_console = console).log.apply(_console, ['GATRACK'].concat(args)); //eslint-disable-line no-console

      }
    },
    isNullOrEnforced: function isNullOrEnforced(provider) {
      return GATrack.force === null || GATrack.force === provider;
    },

    get isGAQ() {
      // eslint-disable-line no-underscore-dangle
      return typeof window._gaq !== 'undefined' && GATrack.isNullOrEnforced('gaq');
    },

    get isGTag() {
      return typeof window.gtag !== 'undefined' && GATrack.isNullOrEnforced('gtag');
    },

    get isGA() {
      return typeof window.ga !== 'undefined' && GATrack.isNullOrEnforced('ga');
    },

    get isEnabled() {
      return GATrack.isGA || GATrack.isGTag || GATrack.isGAQ;
    },

    debug: _typeof$1(window.localStorage) === 'object' && window.localStorage.getItem('GATrackDebug'),
    prefix: null,
    force: null,
    defaults: {
      timeout: 1000
    }
  };

  var outlineTracked = function outlineTracked() {
    return find$1('[data-ga-track-initialised], [data-ga-track]').map(function (el) {
      el.style.outline = 'red dotted 1px';
      return el;
    });
  };

  GATrack.debug = window.localStorage.getItem('GATrackDebug');
  var outline = window.localStorage.getItem('GATrackOutline');
  window.GAOutlineTracked = outlineTracked;
  ready(function () {
    if (!window._GATrack_) {
      window._GATrack_ = GATrack;
      GATrack.autotrack();

      if (outline) {
        outlineTracked();
      }
    }
  });

  function isWindow$2(obj) {
    return obj != null && obj === obj.window;
  }

  function find$2(selector) {
    var context = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;

    if (selector instanceof HTMLElement || selector instanceof Node || isWindow$2(selector)) {
      return [selector];
    } else if (selector instanceof NodeList) {
      return [].slice.call(selector);
    } else if (typeof selector === 'string') {
      var startElement = context ? find$2(context)[0] : document;
      return [].slice.call(startElement.querySelectorAll(selector));
    }

    return [];
  }

  var NativeCustomEvent$2 = window.CustomEvent; //
  // Check for the usage of native support for CustomEvents which is lacking
  // completely on IE.
  //

  function canIuseNativeCustom$2() {
    try {
      var p = new NativeCustomEvent$2('t', {
        detail: {
          a: 'b'
        }
      });
      return p.type === 't' && p.detail.a === 'b';
    } catch (e) {
      return false;
    }
  } // Lousy polyfill for the Custom Event constructor for IE.


  var IECustomEvent$2 = function CustomEvent(type, params) {
    var e = document.createEvent('CustomEvent');

    if (params) {
      e.initCustomEvent(type, params.bubbles, params.cancelable, params.detail);
    } else {
      e.initCustomEvent(type, false, false, undefined);
    }

    return e;
  };

  var DomassistCustomEvent$2 = canIuseNativeCustom$2() ? NativeCustomEvent$2 : IECustomEvent$2;

  function fire(selector, type) {
    var params = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

    if (Array.isArray(selector)) {
      return selector.forEach(function (item) {
        return fire(item, type, params);
      });
    }

    var els = find$2(selector);

    if (els.length) {
      if (params.bubbles !== false) {
        params.bubbles = true;
      }

      els.forEach(function (el) {
        var event = new DomassistCustomEvent$2(type, params);
        el.dispatchEvent(event);
      });
      return els;
    }
  }

  var SCROLLABLE_CONTAINER$2;

  function getScrollableContainer$2() {
    if (SCROLLABLE_CONTAINER$2) {
      return SCROLLABLE_CONTAINER$2;
    }

    var documentElement = window.document.documentElement;
    var scrollableContainer;
    documentElement.scrollTop = 1;

    if (documentElement.scrollTop === 1) {
      documentElement.scrollTop = 0;
      scrollableContainer = documentElement;
    } else {
      scrollableContainer = document.body;
    }

    SCROLLABLE_CONTAINER$2 = scrollableContainer;
    return scrollableContainer;
  }

  SCROLLABLE_CONTAINER$2 = getScrollableContainer$2();

  var setupReady$1 = function setupReady(callbacks) {
    return function (callback) {
      callbacks.push(callback);

      function execute() {
        while (callbacks.length) {
          var fn = callbacks.shift();

          if (typeof fn === 'function') {
            fn();
          }
        }
      }

      function loaded() {
        document.removeEventListener('DOMContentLoaded', loaded);
        execute();
      }

      setTimeout(function () {
        if (document.readyState !== 'loading') {
          return execute();
        }
      }, 0);
      document.addEventListener('DOMContentLoaded', loaded);
    };
  };

  var ready$1 = setupReady$1([]);

  var scrollPercent = 0;
  var wHeight = 0;
  var dHeight = 0;
  var trackLength = 0;
  var location = document.location.toString();
  var hasScrolled = false;
  var cache = [25, 50, 75, 100];
  var Events = {
    Scroll: 'user:scroll'
  };

  function getDocHeight() {
    return Math.max(document.body.scrollHeight, document.documentElement.scrollHeight, document.body.offsetHeight, document.documentElement.offsetHeight, document.body.clientHeight, document.documentElement.clientHeight);
  }

  function computeSizes() {
    wHeight = window.innerHeight || (document.documentElement || document.body).clientHeight;
    dHeight = getDocHeight();
    trackLength = dHeight - wHeight;
  }

  function getScrolledAmount() {
    var scrollTop = window.pageYOffset || (document.documentElement || document.body.parentNode || document.body).scrollTop;
    return Math.floor(scrollTop / trackLength * 100);
  }

  function trackScrollPosition(e) {
    var category = 'scroll';
    var action = location;
    var label = "Scrolled ".concat(e, "%");
    var value = e;

    if (GATrack.isGTag) {
      // Gtag check needs to go before since gtag creates a ga variable
      var payload = {
        event_category: category,
        event_label: label,
        value: value
      };
      GATrack.sendData('event', action, payload);
    } else {
      GATrack.sendData('send', 'event', category, action, label, value);
    }

    fire(document.body, Events.Scroll, {
      detail: {
        amount: e
      }
    });
  }

  function scrollCheck() {
    if (!cache.length) {
      return;
    }

    scrollPercent = getScrolledAmount();

    if (!hasScrolled && scrollPercent > 0) {
      hasScrolled = true;
      GATrack.sendEvent('scroll', location, 'Scrolled');
    }

    var events = cache.filter(function (a) {
      return a <= scrollPercent;
    });
    events.forEach(function (e) {
      trackScrollPosition(e);
      cache.splice(cache.indexOf(e), 1);
    });
  }

  ready$1(function () {
    if (window.disableGATrackScroll !== true) {
      computeSizes();
      index.on(scrollCheck);
      window.addEventListener('resize', computeSizes, false);
    }
  });

  exports.Events = Events;
  exports.trackScrollPosition = trackScrollPosition;

  return exports;

}({}));
//# sourceMappingURL=ga-track-scroll.js.map
